@@ -270,20 +270,26 @@ static BUFFY **find_buffy_slot (const char *path)
 }
 
 /* To avoid overwriting existing values:
- * - label will be NULL if unspecified
- * - nopoll will be -1 if unspecified
+ * - label should be NULL if unspecified
+ * - nopoll should be -1 if unspecified
+ * - nonotify should be -1 if unspecified
  */
-static void buffy_add (BUFFER *path, const char *label, int nopoll)
+void mutt_buffy_add (const char *path, const char *label, int nopoll, int nonotify)
 {
   BUFFY **tmp;
   struct stat sb;
   int new = 0;
 
-  tmp = find_buffy_slot (mutt_b2s (path));
+  if (!path || !*path)
+    return;
+
+  dprint (3, (debugfile, "mutt_buffy_add: %s\n", path));
+
+  tmp = find_buffy_slot (path);
   if (!*tmp)
   {
     new = 1;
-    *tmp = buffy_new (mutt_b2s (path));
+    *tmp = buffy_new (path);
 #ifdef USE_SIDEBAR
     mutt_sb_notify_mailbox (*tmp, 1);
 #endif
@@ -309,6 +315,9 @@ static void buffy_add (BUFFER *path, const char *label, int nopoll)
 #endif
   }
 
+  if (nonotify != -1)
+      (*tmp)->nonotify = nonotify;
+
   (*tmp)->new = 0;
   (*tmp)->notified = 1;
   (*tmp)->newly_created = 0;
@@ -347,12 +356,26 @@ static void buffy_remove (BUFFY **pbuffy)
   *pbuffy = next;
 }
 
+void mutt_buffy_remove (const char *path)
+{
+  BUFFY **pbuffy;
+
+  if (!path || !*path)
+    return;
+
+  dprint (3, (debugfile, "mutt_buffy_remove: %s\n", path));
+
+  pbuffy = find_buffy_slot (path);
+  if (*pbuffy)
+    buffy_remove (pbuffy);
+}
+
 int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, union pointer_long_t udata,
                           BUFFER *err)
 {
   BUFFER *label = NULL;
   BUFFER *mailbox = NULL;
-  int nopoll = -1, rc = -1;
+  int nonotify = -1, nopoll = -1, rc = -1;
   int label_set = 0, mailbox_set = 0;
 
   mailbox = mutt_buffer_pool_get ();
@@ -368,6 +391,10 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, union pointer_long_t udata,
         nopoll = 0;
       else if (mutt_strcmp (mutt_b2s (path), "-nopoll") == 0)
         nopoll = 1;
+      else if (mutt_strcmp (mutt_b2s (path), "-notify") == 0)
+        nonotify = 0;
+      else if (mutt_strcmp (mutt_b2s (path), "-nonotify") == 0)
+        nonotify = 1;
       else if (mutt_strcmp (mutt_b2s (path), "-label") == 0)
       {
         if (!MoreArgs (s))
@@ -401,7 +428,8 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, union pointer_long_t udata,
       }
     }
     else
-      buffy_add (mailbox, label_set ? mutt_b2s (label) : NULL, nopoll);
+      mutt_buffy_add (mutt_b2s (mailbox), label_set ? mutt_b2s (label) : NULL,
+                      nopoll, nonotify);
 
     mutt_buffer_clear (mailbox);
     mutt_buffer_clear (label);
@@ -746,8 +774,14 @@ int mutt_buffy_check (int force)
 
     if (!tmp->new)
       tmp->notified = 0;
-    else if (!tmp->notified)
-      BuffyNotify++;
+    else
+    {
+      /* pretend we've already notified for the mailbox */
+      if (tmp->nonotify)
+        tmp->notified = 1;
+      else if (!tmp->notified)
+        BuffyNotify++;
+    }
   }
 
   BuffyDoneTime = BuffyTime;
