@@ -816,6 +816,11 @@ static int trash_append (CONTEXT *ctx)
         if (mutt_append_message (&ctx_trash, ctx, ctx->hdrs[i], 0, 0) == -1)
         {
           mx_close_mailbox (&ctx_trash, NULL);
+          /* L10N:
+             Displayed if appending to $trash fails when syncing or closing
+             a mailbox.
+          */
+          mutt_error _("Unable to append to trash folder");
           return -1;
         }
       }
@@ -1380,8 +1385,12 @@ int mx_check_mailbox (CONTEXT *ctx, int *index_hint)
   return ctx->mx_ops->check (ctx, index_hint);
 }
 
-/* return a stream pointer for a message */
-MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
+/* return a stream pointer for a message.
+ *
+ * if headers is set, some backends will only download and return the
+ * message headers.
+ */
+MESSAGE *mx_open_message (CONTEXT *ctx, int msgno, int headers)
 {
   MESSAGE *msg;
 
@@ -1392,7 +1401,7 @@ MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
   }
 
   msg = safe_calloc (1, sizeof (MESSAGE));
-  if (ctx->mx_ops->open_msg (ctx, msg, msgno))
+  if (ctx->mx_ops->open_msg (ctx, msg, msgno, headers))
     FREE (&msg);
 
   return msg;
