@@ -54,7 +54,7 @@ static const char *cache_id(const char *id)
   static char clean[SHORT_STRING];
 
   strfcpy (clean, id, sizeof(clean));
-  mutt_sanitize_filename (clean, 1);
+  mutt_sanitize_filename (clean, 0);
 
   return clean;
 }
@@ -83,22 +83,24 @@ static int pop_read_header (POP_DATA *pop_data, HEADER *h)
 {
   FILE *f;
   int ret, index;
-  long length;
+  LOFF_T length;
   char buf[LONG_STRING];
-  char tempfile[_POSIX_PATH_MAX];
+  BUFFER *tempfile;
 
-  mutt_mktemp (tempfile, sizeof (tempfile));
-  if (!(f = safe_fopen (tempfile, "w+")))
+  tempfile = mutt_buffer_pool_get ();
+  mutt_buffer_mktemp (tempfile);
+  if (!(f = safe_fopen (mutt_b2s (tempfile), "w+")))
   {
-    mutt_perror (tempfile);
-    return -3;
+    mutt_perror (mutt_b2s (tempfile));
+    ret = -3;
+    goto cleanup;
   }
 
   snprintf (buf, sizeof (buf), "LIST %d\r\n", h->refno);
   ret = pop_query (pop_data, buf, sizeof (buf));
   if (ret == 0)
   {
-    sscanf (buf, "+OK %d %ld", &index, &length);
+    sscanf (buf, "+OK %d " OFF_T_FMT, &index, &length);
 
     snprintf (buf, sizeof (buf), "TOP %d 0\r\n", h->refno);
     ret = pop_fetch_data (pop_data, buf, NULL, fetch_message, f);
@@ -151,7 +153,10 @@ static int pop_read_header (POP_DATA *pop_data, HEADER *h)
   }
 
   safe_fclose (&f);
-  unlink (tempfile);
+  unlink (mutt_b2s (tempfile));
+
+cleanup:
+  mutt_buffer_pool_release (&tempfile);
   return ret;
 }
 
@@ -228,9 +233,9 @@ static int msg_cache_check (const char *id, body_cache_t *bcache, void *data)
 }
 
 #ifdef USE_HCACHE
-static int pop_hcache_namer (const char *path, char *dest, size_t destlen)
+static void pop_hcache_namer (const char *path, BUFFER *dest)
 {
-  return snprintf (dest, destlen, "%s." HC_FEXT, path);
+  mutt_buffer_printf (dest, "%s." HC_FEXT, path);
 }
 
 static header_cache_t *pop_hcache_open (POP_DATA *pop_data, const char *path)
@@ -537,12 +542,12 @@ int pop_close_mailbox (CONTEXT *ctx)
 }
 
 /* fetch message from POP server */
-static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
+static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno, int headers)
 {
-  int ret;
+  int ret, rc = -1;
   void *uidl;
   char buf[LONG_STRING];
-  char path[_POSIX_PATH_MAX];
+  BUFFER *path = NULL;
   progress_t progressbar;
   POP_DATA *pop_data = (POP_DATA *)ctx->data;
   POP_CACHE *cache;
@@ -572,7 +577,7 @@ static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
       mutt_sleep (2);
       return -1;
     }
-    else
+    else if (!headers)
     {
       /* clear the previous entry */
       unlink (cache->path);
@@ -580,75 +585,96 @@ static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
     }
   }
 
+  path = mutt_buffer_pool_get ();
+
   FOREVER
   {
     if (pop_reconnect (ctx) < 0)
-      return -1;
+      goto cleanup;
 
     /* verify that massage index is correct */
     if (h->refno < 0)
     {
       mutt_error _("The message index is incorrect. Try reopening the mailbox.");
       mutt_sleep (2);
-      return -1;
+      goto cleanup;
     }
 
     mutt_progress_init (&progressbar, _("Fetching message..."),
-			MUTT_PROGRESS_SIZE, NetInc, h->content->length + h->content->offset - 1);
+			MUTT_PROGRESS_SIZE, NetInc,
+                        headers ?
+                        h->content->offset - 1 :
+                        h->content->length + h->content->offset - 1);
 
     /* see if we can put in body cache; use our cache as fallback */
-    if (!(msg->fp = mutt_bcache_put (pop_data->bcache, cache_id (h->data), 1)))
+    if (headers ||
+        !(msg->fp = mutt_bcache_put (pop_data->bcache, cache_id (h->data), 1)))
     {
       /* no */
       bcache = 0;
-      mutt_mktemp (path, sizeof (path));
-      if (!(msg->fp = safe_fopen (path, "w+")))
+      mutt_buffer_mktemp (path);
+      if (!(msg->fp = safe_fopen (mutt_b2s (path), "w+")))
       {
-	mutt_perror (path);
+	mutt_perror (mutt_b2s (path));
 	mutt_sleep (2);
-	return -1;
+	goto cleanup;
       }
-    }
 
-    snprintf (buf, sizeof (buf), "RETR %d\r\n", h->refno);
+      if (headers)
+        unlink (mutt_b2s (path));
+   }
+
+    snprintf (buf, sizeof (buf),
+              headers ? "TOP %d 0\r\n" : "RETR %d\r\n",
+              h->refno);
 
     ret = pop_fetch_data (pop_data, buf, &progressbar, fetch_message, msg->fp);
     if (ret == 0)
+    {
+      if (headers && pop_data->cmd_top == 2)
+	pop_data->cmd_top = 1;
       break;
+    }
 
     safe_fclose (&msg->fp);
 
     /* if RETR failed (e.g. connection closed), be sure to remove either
      * the file in bcache or from POP's own cache since the next iteration
      * of the loop will re-attempt to put() the message */
-    if (!bcache)
-      unlink (path);
+    if (!bcache && !headers)
+      unlink (mutt_b2s (path));
 
     if (ret == -2)
     {
+      if (headers && pop_data->cmd_top == 2)
+        pop_data->cmd_top = 0;
       mutt_error ("%s", pop_data->err_msg);
       mutt_sleep (2);
-      return -1;
+      goto cleanup;
     }
 
     if (ret == -3)
     {
       mutt_error _("Can't write message to temporary file!");
       mutt_sleep (2);
-      return -1;
+      goto cleanup;
     }
   }
 
   /* Update the header information.  Previously, we only downloaded a
    * portion of the headers, those required for the main display.
    */
-  if (bcache)
-    mutt_bcache_commit (pop_data->bcache, cache_id (h->data));
-  else
+  if (!headers)
   {
-    cache->index = h->index;
-    cache->path = safe_strdup (path);
+    if (bcache)
+      mutt_bcache_commit (pop_data->bcache, cache_id (h->data));
+    else
+    {
+      cache->index = h->index;
+      cache->path = safe_strdup (mutt_b2s (path));
+    }
   }
+
   rewind (msg->fp);
   uidl = h->data;
 
@@ -663,15 +689,19 @@ static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
   mutt_label_hash_add (ctx, h);
 
   h->data = uidl;
-  h->lines = 0;
-  fgets (buf, sizeof (buf), msg->fp);
-  while (!feof (msg->fp))
-  {
-    ctx->hdrs[msgno]->lines++;
-    fgets (buf, sizeof (buf), msg->fp);
-  }
 
-  h->content->length = ftello (msg->fp) - h->content->offset;
+  if (!headers)
+  {
+    h->lines = 0;
+    fgets (buf, sizeof (buf), msg->fp);
+    while (!feof (msg->fp))
+    {
+      ctx->hdrs[msgno]->lines++;
+      fgets (buf, sizeof (buf), msg->fp);
+    }
+
+    h->content->length = ftello (msg->fp) - h->content->offset;
+  }
 
   /* This needs to be done in case this is a multipart message */
   if (!WithCrypto)
@@ -680,7 +710,11 @@ static int pop_fetch_message (CONTEXT* ctx, MESSAGE* msg, int msgno)
   mutt_clear_error();
   rewind (msg->fp);
 
-  return 0;
+  rc = 0;
+
+cleanup:
+  mutt_buffer_pool_release (&path);
+  return rc;
 }
 
 static int pop_close_message (CONTEXT *ctx, MESSAGE *msg)
