@@ -134,7 +134,8 @@ int mutt_num_postponed (int force)
 
     if (access (Postponed, R_OK | F_OK) != 0)
       return (PostCount = 0);
-    if (mx_open_mailbox (Postponed, MUTT_NOSORT | MUTT_QUIET, &ctx) == NULL)
+    if (mx_open_mailbox (Postponed, MUTT_NOSORT | MUTT_QUIET | MUTT_READONLY,
+                         &ctx) == NULL)
       PostCount = 0;
     else
       PostCount = ctx.msgcount;
@@ -223,20 +224,15 @@ static HEADER *select_msg (void)
 /* args:
  *      ctx	Context info, used when recalling a message to which
  *              we reply.
- *	hdr	envelope/attachment info for recalled message
- *	cur	if message was a reply, `cur' is set to the message which
- *		`hdr' is in reply to
- *	fcc	fcc for the recalled message
+ *      sctx    Send Context info.
  *
  * return vals:
  *	-1		error/no messages
  *	0		normal exit
- *	SENDREPLY	recalled message is a reply
  */
-int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
+int mutt_get_postponed (CONTEXT *ctx, SEND_CONTEXT *sctx)
 {
   HEADER *h;
-  int code = SENDPOSTPONED;
   LIST *tmp;
   LIST *last = NULL;
   LIST *next;
@@ -254,18 +250,6 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
     return (-1);
   }
 
-  /* TODO:
-   * mx_open_mailbox() for IMAP leaves IMAP_REOPEN_ALLOW set.  For the
-   * index this is papered-over because it calls mx_check_mailbox()
-   * every event loop (which resets that flag).
-   *
-   * For a stable-branch fix, I'm doing the same here, to prevent
-   * context changes from occuring behind the scenes and causing
-   * segvs, but probably the flag needs to be reset after downloading
-   * headers in imap_open_mailbox().
-   */
-  mx_check_mailbox (PostContext, NULL);
-
   if (! PostContext->msgcount)
   {
     PostCount = 0;
@@ -293,7 +277,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
     return (-1);
   }
 
-  if (mutt_prepare_template (NULL, PostContext, hdr, h, 0) < 0)
+  if (mutt_prepare_template (NULL, PostContext, sctx->msg, h, 0) < 0)
   {
     mx_fastclose_mailbox (PostContext);
     FREE (&PostContext);
@@ -319,7 +303,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
 
   FREE (&PostContext);
 
-  for (tmp = hdr->env->userhdrs; tmp; )
+  for (tmp = sctx->msg->env->userhdrs; tmp; )
   {
     if (ascii_strncasecmp ("X-Mutt-References:", tmp->data, 18) == 0)
     {
@@ -330,7 +314,14 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
 	p = skip_email_wsp(tmp->data + 18);
 	if (!ctx->id_hash)
 	  ctx->id_hash = mutt_make_id_hash (ctx);
-	*cur = hash_find (ctx->id_hash, p);
+	sctx->cur = hash_find (ctx->id_hash, p);
+        if (sctx->cur)
+        {
+          sctx->has_cur = 1;
+          sctx->cur_message_id = safe_strdup (sctx->cur->env->message_id);
+          sctx->cur_security = sctx->cur->security;
+          sctx->flags |= SENDREPLY;
+        }
       }
 
       /* Remove the X-Mutt-References: header field. */
@@ -338,25 +329,23 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
       if (last)
 	last->next = tmp->next;
       else
-	hdr->env->userhdrs = tmp->next;
+	sctx->msg->env->userhdrs = tmp->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
-      if (*cur)
-	code |= SENDREPLY;
     }
     else if (ascii_strncasecmp ("X-Mutt-Fcc:", tmp->data, 11) == 0)
     {
       p = skip_email_wsp(tmp->data + 11);
-      mutt_buffer_strcpy (fcc, p);
-      mutt_buffer_pretty_mailbox (fcc);
+      mutt_buffer_strcpy (sctx->fcc, p);
+      mutt_buffer_pretty_multi_mailbox (sctx->fcc, FccDelimiter);
 
       /* remove the X-Mutt-Fcc: header field */
       next = tmp->next;
       if (last)
 	last->next = tmp->next;
       else
-	hdr->env->userhdrs = tmp->next;
+	sctx->msg->env->userhdrs = tmp->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
@@ -365,7 +354,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
        * user to not make a copy if the header field is present, but empty.
        * see http://dev.mutt.org/trac/ticket/3653
        */
-      code |= SENDPOSTPONEDFCC;
+      sctx->flags |= SENDPOSTPONEDFCC;
     }
     else if ((WithCrypto & APPLICATION_PGP)
              && (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
@@ -373,16 +362,16 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
                                                            */
                  || mutt_strncmp ("X-Mutt-PGP:", tmp->data, 11) == 0))
     {
-      hdr->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1,
-					    APPLICATION_PGP);
-      hdr->security |= APPLICATION_PGP;
+      sctx->msg->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1,
+                                                  APPLICATION_PGP, sctx);
+      sctx->msg->security |= APPLICATION_PGP;
 
       /* remove the pgp field */
       next = tmp->next;
       if (last)
 	last->next = tmp->next;
       else
-	hdr->env->userhdrs = tmp->next;
+	sctx->msg->env->userhdrs = tmp->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
@@ -390,16 +379,16 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
     else if ((WithCrypto & APPLICATION_SMIME)
              && mutt_strncmp ("X-Mutt-SMIME:", tmp->data, 13) == 0)
     {
-      hdr->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1,
-					    APPLICATION_SMIME);
-      hdr->security |= APPLICATION_SMIME;
+      sctx->msg->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1,
+                                                  APPLICATION_SMIME, sctx);
+      sctx->msg->security |= APPLICATION_SMIME;
 
       /* remove the smime field */
       next = tmp->next;
       if (last)
 	last->next = tmp->next;
       else
-	hdr->env->userhdrs = tmp->next;
+	sctx->msg->env->userhdrs = tmp->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
@@ -409,12 +398,12 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
     else if (mutt_strncmp ("X-Mutt-Mix:", tmp->data, 11) == 0)
     {
       char *t;
-      mutt_free_list (&hdr->chain);
+      mutt_free_list (&sctx->msg->chain);
 
       t = strtok (tmp->data + 11, " \t\n");
       while (t)
       {
-	hdr->chain = mutt_add_list (hdr->chain, t);
+	sctx->msg->chain = mutt_add_list (sctx->msg->chain, t);
 	t = strtok (NULL, " \t\n");
       }
 
@@ -422,7 +411,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
       if (last)
 	last->next = tmp->next;
       else
-	hdr->env->userhdrs = tmp->next;
+	sctx->msg->env->userhdrs = tmp->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
       tmp = next;
@@ -437,14 +426,15 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, BUFFER *fcc)
   }
 
   if (option (OPTCRYPTOPPORTUNISTICENCRYPT))
-    crypt_opportunistic_encrypt (hdr);
+    crypt_opportunistic_encrypt (sctx->msg);
 
-  return (code);
+  return (0);
 }
 
 
 
-int mutt_parse_crypt_hdr (const char *p, int set_empty_signas, int crypt_app)
+int mutt_parse_crypt_hdr (const char *p, int set_empty_signas, int crypt_app,
+                          SEND_CONTEXT *sctx)
 {
   char smime_cryptalg[LONG_STRING] = "\0";
   char sign_as[LONG_STRING] = "\0", *q;
@@ -560,19 +550,19 @@ int mutt_parse_crypt_hdr (const char *p, int set_empty_signas, int crypt_app)
 
   /* the cryptalg field must not be empty */
   if ((WithCrypto & APPLICATION_SMIME) && *smime_cryptalg)
-    mutt_str_replace (&SmimeCryptAlg, smime_cryptalg);
+    mutt_str_replace (&sctx->smime_crypt_alg, smime_cryptalg);
 
   /* Set {Smime,Pgp}SignAs, if desired. */
 
   if ((WithCrypto & APPLICATION_PGP) && (crypt_app == APPLICATION_PGP)
       && (flags & SIGN)
       && (set_empty_signas || *sign_as))
-    mutt_str_replace (&PgpSignAs, sign_as);
+    mutt_str_replace (&sctx->pgp_sign_as, sign_as);
 
   if ((WithCrypto & APPLICATION_SMIME) && (crypt_app == APPLICATION_SMIME)
       && (flags & SIGN)
       && (set_empty_signas || *sign_as))
-    mutt_str_replace (&SmimeSignAs, sign_as);
+    mutt_str_replace (&sctx->smime_sign_as, sign_as);
 
   return flags;
 }
@@ -601,7 +591,7 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
 
   memset (&s, 0, sizeof (s));
 
-  if (!fp && (msg = mx_open_message (ctx, hdr->msgno)) == NULL)
+  if (!fp && (msg = mx_open_message (ctx, hdr->msgno, 0)) == NULL)
     return (-1);
 
   if (!fp) fp = msg->fp;
@@ -624,7 +614,7 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   if (resend)
   {
     FREE (&newhdr->env->message_id);
-    FREE (&newhdr->env->mail_followup_to);
+    rfc822_free_address (&newhdr->env->mail_followup_to);
   }
 
   /* decrypt pgp/mime encoded messages */
