@@ -189,9 +189,9 @@ static const char *cb_format_str(char *dest, size_t destlen, size_t col, int col
   }
 
   if (optional)
-    mutt_FormatString (dest, destlen, col, SidebarWidth, ifstring,   cb_format_str, sbe, flags);
+    mutt_FormatString (dest, destlen, col, cols, ifstring,   cb_format_str, sbe, flags);
   else if (flags & MUTT_FORMAT_OPTIONAL)
-    mutt_FormatString (dest, destlen, col, SidebarWidth, elsestring, cb_format_str, sbe, flags);
+    mutt_FormatString (dest, destlen, col, cols, elsestring, cb_format_str, sbe, flags);
 
   /* We return the format string, unchanged */
   return src;
@@ -499,7 +499,7 @@ static int draw_divider (int num_rows, int num_cols)
   int i;
   for (i = 0; i < num_rows; i++)
   {
-    mutt_window_move (MuttSidebarWindow, i, SidebarWidth - delim_len);	//RAR 0 for rhs
+    mutt_window_move (MuttSidebarWindow, i, num_cols - delim_len);	//RAR 0 for rhs
     addstr (NONULL(SidebarDividerChar));
   }
 
@@ -626,6 +626,7 @@ static void calculate_depth (const char *path, const char *lastpath,
 static void draw_sidebar (int num_rows, int num_cols, int div_width)
 {
   int entryidx;
+  int entry_color;
   SBENTRY *entry;
   BUFFY *b;
   int maildir_is_prefix;
@@ -641,7 +642,7 @@ static void draw_sidebar (int num_rows, int num_cols, int div_width)
   last_folder_name = mutt_buffer_pool_get ();
   indent_folder_name = mutt_buffer_pool_get ();
 
-  int w = MIN(num_cols, (SidebarWidth - div_width));
+  int w = num_cols - div_width;
   int row = 0;
   for (entryidx = TopIndex; (entryidx < EntryCount) && (row < num_rows); entryidx++)
   {
@@ -650,24 +651,32 @@ static void draw_sidebar (int num_rows, int num_cols, int div_width)
       continue;
     b = entry->buffy;
 
+    if ((b->msg_unread > 0) || (b->new))
+      entry_color = MT_COLOR_NEW;
+    else if (b->msg_flagged > 0)
+      entry_color = MT_COLOR_FLAGGED;
+    else if ((ColorDefs[MT_COLOR_SB_SPOOLFILE].pair != 0 ||
+              ColorDefs[MT_COLOR_SB_SPOOLFILE].attrs != 0) &&
+             (mutt_strcmp (mutt_b2s (b->pathbuf), Spoolfile) == 0))
+      entry_color = MT_COLOR_SB_SPOOLFILE;
+    else
+      entry_color = MT_COLOR_NORMAL;
+
     if (entryidx == OpnIndex)
     {
-      if ((ColorDefs[MT_COLOR_SB_INDICATOR] != 0))
-        SETCOLOR(MT_COLOR_SB_INDICATOR);
+      if ((ColorDefs[MT_COLOR_SB_INDICATOR].pair != 0 ||
+           ColorDefs[MT_COLOR_SB_INDICATOR].attrs != 0))
+	mutt_attrset_cursor (ColorDefs[entry_color],
+		ColorDefs[MT_COLOR_SB_INDICATOR]);
       else
-        SETCOLOR(MT_COLOR_INDICATOR);
+	mutt_attrset_cursor (ColorDefs[entry_color],
+		ColorDefs[MT_COLOR_INDICATOR]);
     }
     else if (entryidx == HilIndex)
-      SETCOLOR(MT_COLOR_HIGHLIGHT);
-    else if ((b->msg_unread > 0) || (b->new))
-      SETCOLOR(MT_COLOR_NEW);
-    else if (b->msg_flagged > 0)
-      SETCOLOR(MT_COLOR_FLAGGED);
-    else if ((ColorDefs[MT_COLOR_SB_SPOOLFILE] != 0) &&
-             (mutt_strcmp (mutt_b2s (b->pathbuf), Spoolfile) == 0))
-      SETCOLOR(MT_COLOR_SB_SPOOLFILE);
+      mutt_attrset_cursor (ColorDefs[entry_color],
+	      ColorDefs[MT_COLOR_HIGHLIGHT]);
     else
-      SETCOLOR(MT_COLOR_NORMAL);
+      SETCOLOR(entry_color);
 
     mutt_window_move (MuttSidebarWindow, row, 0);
     if (Context && Context->realpath && !b->nopoll &&
@@ -705,8 +714,17 @@ static void draw_sidebar (int num_rows, int num_cols, int div_width)
         sidebar_folder_name = mutt_b2s (b->pathbuf) + (maildirlen + 1);
         maildir_is_prefix = 1;
       }
+      /* mutt_expand_path() now expands relative paths too.
+       * To try not to change the sidebar in case it has those, use
+       * pretty_mailbox to translate back to relative paths */
       else
-        sidebar_folder_name = mutt_b2s (b->pathbuf);
+      {
+        mutt_buffer_strcpy (pretty_folder_name, mutt_b2s (b->pathbuf));
+        mutt_buffer_pretty_mailbox (pretty_folder_name);
+        sidebar_folder_name = mutt_b2s (pretty_folder_name);
+        if ((*sidebar_folder_name == '=') || (*sidebar_folder_name == '~'))
+          sidebar_folder_name = mutt_b2s (b->pathbuf);
+      }
     }
 
     if (SidebarDelimChars)
@@ -811,12 +829,15 @@ void mutt_sb_draw (void)
   int num_rows  = MuttSidebarWindow->rows;
   int num_cols  = MuttSidebarWindow->cols;
 
+  if (!num_cols)
+    return;
+
   int div_width = draw_divider (num_rows, num_cols);
-  if (div_width < 0)
+  if ((div_width < 0) || (div_width >= num_cols))
     return;
 
   if (!prepare_sidebar (num_rows))
-    fill_empty_space (0, num_rows, SidebarWidth - div_width);
+    fill_empty_space (0, num_rows, num_cols - div_width);
   else
     draw_sidebar (num_rows, num_cols, div_width);
 }
